      
c******************************************************************
      
      subroutine figures2(mtot,stot,vrot,m,s,ro,dros,n,ndis,idisc,
     &     deltaro,a1,j2,j4,b,h)
c     Version: 02/12/99
      
c     Auteur: Tristan Guillot - Version du 18/05/92
      
c Entrees:
c     mtot: masse totale
c     stot: rayon total
c     vrot: vitesse angulaire de rotation [rad/s]
c     m: tableau des masses (normalises - m(1)=1.; m(n)=0.)
c     s: tableau des rayons moyens (normalises - r(1)=1.; r(n)=0.)
c     ro: densites en g/cm3
c     dros: d(ro)/ds  [g/cm3]
c     n: nombre de couches du modele
c     ndis: nombre de discontinuites (min=1)
c     idisc: tableau des couches des discontinuites
c            (couche inferieure: idisc(idis)-1; couche superieure: idisc(idis))
c     deltaro: tableau des sauts de densite (densite couche sup - couche inf)
c Sorties:
c     a1: rayon equatorial
c     j2,j4: potentiels gravitationnels
c     b(n),h(n): tableaux des corrections au potentiel dues a la rotation

c---- Exemple de discontinuite: ------------------------------------------
c     idis=1 -> transition MIP/surface
c     idis=2: (par ex. PPT)
c           couche      m           s        ro
c             76      0.87185    0.96172   0.91753
c             77      0.86342    0.95618   0.93545   <- couche "inferieure"
c             78      0.86342    0.95618   1.30445   <- couche "superieure"
c             79      0.85950    0.95121   1.31789
c     ==> idisc(2)=78; deltaro(2)=+0.36900
c--------------------------------------------------------------------------

      implicit none
      
      include '/users/user2/guillot/Cepam/cepam.inc'
      integer pn_d
      parameter (pn_d=pn+pn_noy+pndis)
      
      integer n,ndis,idisc(ndis)
      real*8 mtot,stot,vrot,m(n),s(n),ro(n),dros(n),
     &     deltaro(ndis),b(n),h(n)
      
      integer i,j,iter,iter2,iter_max,ncouche,m_S
      data iter_max/50/
      real*8 ss0(pn_d),mm0(pn_d),v0,a1,old_a1,ds,mm,er,err,ss,romoy,
     &     s2(pn_d),ss2(pn_d),tt2(pn_d),j2,new_s2,somme2,
     &     dsom2,old_j2,vs2,vss2,vtt2,
     &     s4(pn_d),ss4(pn_d),tt4(pn_d),j4,new_s4,somme4,
     &     dsom4,old_j4,vs4,vss4,vtt4
      real*8 aerr0,prec,precix
      data prec/1d-8/,precix/1.d-4/
      
      logical init,rapid,verif
      data init/.true./,verif/.false./
      
      include '/users/user2/guillot/Cepam/Communs/ctephys.inc'
      
cccccccccccccccc Equations de la theorie des figures cccccccccccccccccc
      
      real*8 s0,req,b0,h0,a2,f2,g2,a4,f4,g4
      real*8 m0,x2,aa2,bb2,x4,aa4,bb4
      
      s0(x2,x4)=-1/5.d0*x2**2-2/105.d0*x2**3-1/35.d0*x2**4
     &      -2/35.d0*x2**2*x4-1/9.d0*x4**2
      req(x2,x4)=1+s0(x2,x4)-1/2.d0*x2+3/8.d0*x4

      b0(m0,x2,aa2,bb2,x4,aa4,bb4)=
     &     m0*(-2/15.d0*x2-3/35.d0*x2**2+22/1575.d0*x2**3
     &     -4/105.d0*x2*x4)+
     &     (2/5.d0*x2**2-4/105.d0*x2**3+48/175.d0*x2**4
     &     -4/35.d0*x2**2*x4+2/9.d0*x4**2)+
     &     (-3/5.d0*x2+12/35.d0*x2**2-234/175.d0*x2**3
     &     +24/35.d0*x2*x4)*aa2+
     &     (192/35.d0*x2**2-5/9.d0*x4)*aa4+
     &     (2/5.d0*x2+2/35.d0*x2**2-2/35.d0*x2**3+4*x2*x4/35.d0)*bb2+
     &     (12/35.d0*x2**2+4/9.d0*x4)*bb4

      a2(m0,x2,aa2,bb2,x4,aa4,bb4)=m0*(-1/3.d0+10/21.d0*x2
     &     +3/35.d0*x2**2
     &     -26/315.d0*x2**3-4/21.d0*x4+20/231.d0*x2*x4)+
     &     (2/7.d0*x2**2-29/35.d0*x2**3+454/1155.d0*x2**4+4/7.d0*x2*x4
     &     -36/77.d0*x2**2*x4+100/693.d0*x4**2)+
     &     (1-6/7.d0*x2+111/35.d0*x2**2-1242/385.d0*x2**3-6/7.d0*x4
     &     +144/77.d0*x2*x4)*aa2 +
     &     (-10/7.d0*x2+1152/77.d0*x2**2-500/693.d0*x4)*aa4 +
     &     (1+4/7.d0*x2+1/35.d0*x2**2-16/105.d0*x2**3+4/7.d0*x4
     &     +24/77.d0*x2*x4)*bb2 +
     &     (8/7.d0*x2+72/77.d0*x2*x2+400/693.d0*x4)*bb4
      
      a4(m0,x2,aa2,bb2,x4,aa4,bb4)=m0*(-12/35.d0*x2+6/77.d0*x2**2
     &     +12/175.d0*x2**3+38/77.d0*x4-326/5005.d0*x2*x4)+
     &     (18/35.d0*x2**2-108/385.d0*x2**3+16902/25025.d0*x2**4
     &     +40/77.d0*x2*x4-7369/5005.d0*x2**2*x4+162/1001.d0*x4**2) +
     &     (-54/35.d0*x2+648/385.d0*x2**2-122688/25025.d0*x2**3
     &     -60/77.d0*x4+21468/5005.d0*x2*x4)*aa2 +
     &     (1-100/77.d0*x2+176749/5005.d0*x2**2-810/1001.d0*x4)*aa4 +
     &     (36/35.d0*x2+108/385.d0*x2**2-36/175.d0*x2**3+40/77.d0*x4
     &     +3578/5005.d0*x2*x4)*bb2 +
     &     (1+80/77.d0*x2+1346/1001.d0*x2**2+648/1001.d0*x4)*bb4
      
      f2(x2,x4)=3/5.d0*x2+12/35.d0*x2**2+6/175.d0*x2**3
     &     -184/1925.d0*x2**4
     &     +24/35.d0*x2*x4+216/385.d0*x2**2*x4+40/2321.d0*x4**2
      f4(x2,x4)=18/35.d0*x2**2+36/77.d0*x2**3
     &     +486/5005.d0*x2**4+1/3.d0*x4
     &     +40/77.d0*x2*x4+6943/5005.d0*x2**2*x4+162/1001.d0*x4**2
      
      h0(x2,x4)=-3/10.d0*x2**2-2/35.d0*x2**3-9/350.d0*x2**4
     &     -6/35.d0*x2**2*x4-1/6.d0*x4**2
      g2(x2,x4)=3/5.d0*x2-3/35.d0*x2**2+36/175.d0*x2**3
     &     -17/275.d0*x2**4
     &     -6/35.d0*x2*x4+36/385.d0*x2**2*x4-10/231.d0*x4**2
      g4(x2,x4)=-9/35.d0*x2**2+72/385.d0*x2**3
     &     +1062/2275.d0*x2**4+1/3.d0*x4
     &     -20/77.d0*x2*x4+4579/5005.d0*x2**2*x4-81/1001.d0*x4**2

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      
c...........S.............A...........V..............E..................
      save 
c-----------------------------------------------------------------------
 2000 format(1p8d10.3)
      mm=vrot**2 /(4/3.d0*pi*g)
      romoy=mtot/(4/3.d0*pi*stot**3)
      write(*,222)stot,mtot,romoy,vrot,mm,mm/romoy
      write(*,*)vrot**2*stot**3/g/mtot
 222  format(1x,'stot=',1pd12.5,' mtot=',1pd12.5,' romoy=',1pd12.5,
     &     'vrot=',1pd12.5,' mm=',1pd12.5,'m=',1pd12.5)

      do i=1,n-1
       ss0(i)=romoy*m(i)/s(i)/s(i)/s(i)
       mm0(i)=mm/ss0(i)
      enddo
      ss0(n)=ro(n)
      mm0(n)=mm/ss0(n)
c     write(*,*)'ss0(i)'
c     write(*,2000)(ss0(i),i=1,n)
      
      if (init) then
       init=.false.
       do i=1,n
        s2(i)=-mm0(i)/3.d0
       enddo
       old_a1=0.d0
       old_j2=0.d0
       old_j4=0.d0
c       write(6,*)'n=',n,' Precix, Ordre des splines?'
c       read(*,*)precix,m_S
       precix=1d-6
       m_S=4
      endif
      
      er=1.d10
      iter=0
      do while ((er.gt.precix).and.(iter.lt.iter_max))
       iter=iter+1
       do i=1,n
        err=1.d10
        iter2=0
        do while ((err.gt.prec).and.(iter2.lt.iter_max))
         iter2=iter2+1
         new_s2=a2(mm0(i),s2(i),ss2(i),tt2(i),s4(i),ss4(i),tt4(i))
         new_s4=a4(mm0(i),s2(i),ss2(i),tt2(i),s4(i),ss4(i),tt4(i))
         err=max(abs((new_s2-s2(i))/new_s2),
     &        abs((new_s4-s4(i))/new_s4))
         s2(i)=new_s2
         s4(i)=new_s4
        enddo
       enddo
c      write(*,*)'s2(1),s4(1)=',s2(1),s4(1)
c     write(*,2000)(s4(i),i=1,n)
       
c       write(*,*)'Integration des ss'
c       call integre_ss(m_S,n,s,dros,ndis,idisc,s2,ss2,s4,ss4)
       call integre_ss2(m_S,n,s,dros,ndis,idisc,s2,ss2,s4,ss4)
c --- discontinuites ----
       do i=1,ndis
        do j=idisc(i)-1,1,-1
         ss2(j)=ss2(j)-s(idisc(i)-1)**5*f2(s2(idisc(i)-1),
     &        s4(idisc(i)-1))*deltaro(i)
         ss4(j)=ss4(j)-s(idisc(i)-1)**7*f4(s2(idisc(i)-1),
     &        s4(idisc(i)-1))*deltaro(i)
        enddo
       enddo
c -----------------------
       do i=1,n-1
        ss2(i)=(f2(s2(i),s4(i))*ro(i)-ss2(i)/s(i)**5)/ss0(i)
        ss4(i)=(f4(s2(i),s4(i))*ro(i)-ss4(i)/s(i)**7)/ss0(i)
       enddo
       ss2(n)=f2(s2(n),s4(n))*ro(n)/ss0(n)
       ss4(n)=f4(s2(n),s4(n))*ro(n)/ss0(n)
       
c       write(*,*)'Integration des tt'
c       call integre_tt(m_S,n,s,dros,ndis,idisc,s2,tt2,s4,tt4)
       call integre_tt2(m_S,n,s,dros,ndis,idisc,s2,tt2,s4,tt4)
c --- discontinuites ----
       do i=1,ndis
        do j=idisc(i),n
         tt2(j)=tt2(j)-g2(s2(idisc(i)),s4(idisc(i)))*deltaro(i)
         tt4(j)=tt4(j)-1/s(idisc(i))**2*g4(s2(idisc(i)),
     &        s4(idisc(i)))*deltaro(i)
        enddo
       enddo
c -----------------------
       do i=2,n
        tt2(i)=(-g2(s2(i),s4(i))*ro(i)-tt2(i))/ss0(i)
        tt4(i)=(-g4(s2(i),s4(i))*ro(i)-tt4(i)*s(i)**2)/ss0(i)
       enddo
       tt2(1)=0.d0
       tt4(1)=0.d0
       
       a1=stot*req(s2(1),s4(1))
       j2=-(stot/a1)**2*ss2(1)
       j4=-(stot/a1)**4*ss4(1)
       er=max(abs((a1-old_a1)/a1),abs((j2-old_j2)/j2),
     &      abs((j4-old_j4)/j4))
       old_a1=a1
       old_j2=j2
       old_j4=j4
c       write(*,300)iter,er,a1,j2,j4
 300   format(1x,'iter_fig2:',i3,' err.max:',1pd8.1,' a1=',
     &      pd12.5,' j2=',1pd12.5,' j4=',1pd12.5)
c     pause 'FIGURES2: pause'
      enddo
      
       write(*,300)iter,er,a1,j2,j4

      do i=1,n
       b(i)=ss0(i)/romoy*b0(mm0(i),s2(i),ss2(i),tt2(i),s4(i),ss4(i),
     &      tt4(i))
       h(i)=h0(s2(i),s4(i))/romoy
c       h(i)=mm/romoy*(-2/15.d0*s2(i))
      enddo
      
      open(unit=22,file='bid2.dat',form='formatted',status='unknown')
      write(22,*)n
      do i=1,n
       write(22,201)m(i),s(i),s0(s2(i),s4(i)),s2(i),s4(i),0.d0
c       write(22,200)i,s(i),ss0(i),s2(i),ss2(i),tt2(i)
c     &      ,s4(i),ss4(i),tt4(i)
 200   format(i4,1x,8(1x,1pd10.3))
 201   format(8(1x,1pd12.5))
      enddo
      write(*,*)'J''ai cree le fichier BID2.DAT'
      close(22)

      return
      
      end
