      
c******************************************************************
      
      subroutine figures3(mtot,stot,vrot,m,s,ro,dros,nn,ndis,idisc,
     &     deltaro,a1,j2,j4,j6,b,h,ierr)
c     Version: 16/06/2003
c     Calculs du potentiel centrifuge par la theorie des figures a l'ordre 3
      
c     Auteur: Tristan Guillot
      
c-Entrees:
c     mtot: masse totale
c     stot: rayon total
c     vrot: vitesse angulaire de rotation [rad/s]
c     m: tableau des masses (normalises - m(1)=1.; m(nn)=0.)
c     s: tableau des rayons moyens (normalises - r(1)=1.; r(nn)=0.)
c     ro: densites en g/cm3
c     dros: d(ro)/ds  [g/cm3]
c     nn: nombre de couches du modele
c     ndis: nombre de discontinuites (min=1)
c     idisc: tableau des couches des discontinuites
c     (couche inferieure: idisc(idis)-1; couche superieure: idisc(idis))
c     deltaro: tableau des sauts de densite (densite couche sup - couche inf)
c-Sorties:
c     a1: rayon equatorial
c     j2,j4,j6: potentiels gravitationnels
c     b,h: tableaux des corrections au potentiel dues a la rotation
c     ierr: 0: tvb; 1: pb de convergence
      
c---- Exemple de discontinuite: ------------------------------------------
c     idis=1 -> transition MIP/surface
c     idis=2: (par ex. PPT)
c     couche      m           s        ro
c     76      0.87185    0.96172   0.91753
c     77      0.86342    0.95618   0.93545   <- couche "inferieure"
c     78      0.86342    0.95618   1.30445   <- couche "superieure"
c     79      0.85950    0.95121   1.31789
c     ==> idisc(2)=78; deltaro(2)=+0.36900
c--------------------------------------------------------------------------

c-Remarques:
c     "figures2" peut initialiser le COMMON /fig2/
      
      implicit none
      
      include './cepam.inc'
      integer pn_d
      parameter (pn_d=pn+pn_noy+pndis)
      
      integer nn,ndis,idisc(ndis),ierr
      real*8 mtot,stot,vrot,m(nn),s(nn),ro(nn),dros(nn),
     &     deltaro(ndis),b(nn),h(nn)
      
      integer i,j,iter,iter2,m_Sf
      real*8 ss0(pn_d),mm0(pn_d),
     &     a1,old_a1,mm,er,err,romoy,
     &     s2(pn_d),ss2(pn_d),tt2(pn_d),j2,new_s2,old_j2,
     &     s4(pn_d),ss4(pn_d),tt4(pn_d),j4,new_s4,old_j4,
     &     s6(pn_d),ss6(pn_d),tt6(pn_d),j6,old_j6
      real*8 prec,precif,err2,err4
      
      logical init
      
      include './Communs/modelp.inc'
      include './Communs/ctephys.inc'
      
      common /fig2/old_a1,ss0,s2,ss2,tt2,old_j2,s4,ss4,tt4,old_j4

ccccccccccccccccEquations de la theorie des figures cccccccccccccccccc
      
      real*8 s0,req,b0,h0,a2,f2,g2,a4,f4,g4,a6,f6,g6
      real*8 m0,x2,aa2,bb2,x4,aa4,bb4,x6,aa6,bb6
      
      s0(x2,x4)=-1/35.d0*x2**4-2/35.d0*x2**2*x4-1/9.d0*x4**2
     &     -2/105.d0*x2**3-1/5.d0*x2**2
      req(x2,x4,x6)=-5/16.d0*x6+3/8.d0*x4+s0(x2,x4)-1/2.d0*x2+1.d0
      
      b0(m0,x2,aa2,bb2)=
     &     m0*(-3/35.d0*x2**2-2/15.d0*x2)+
     &     (-4/105.d0*x2**3+2/5.d0*x2**2)+
     &     (12/35.d0*x2**2-3/5.d0*x2)*aa2 +
     &     (2/35.d0*x2**2+2/5.d0*x2)*bb2
      
      a2(m0,x2,aa2,bb2,x4,aa4,bb4)=m0*(3/35.d0*x2**2-4/21.d0*x4
     &     +10/21.d0*x2-1/3.d0)+
     &     (-29/35.d0*x2**3+4/7.d0*x2*x4+2/7.d0*x2**2)+
     &     (111/35.d0*x2**2-6/7.d0*x4-6/7.d0*x2+1.d0)*aa2 +
     &     (-10/7.d0*x2)*aa4 +
     &     (1/35.d0*x2**2+4/7.d0*x4+4/7.d0*x2+1.d0)*bb2 +
     &     (8/7.d0*x2)*bb4
      
      a4(m0,x2,aa2,bb2,x4,aa4,bb4)=
     &     m0*(6/77.d0*x2**2+38/77.d0*x4-12/35.d0*x2)+
     &     (-108/385.d0*x2**3+40/77.d0*x2*x4+18/35.d0*x2**2)+
     &     (648/385.d0*x2**2-60/77.d0*x4-54/35.d0*x2)*aa2 +
     &     (-100/77.d0*x2+1.d0)*aa4 +
     &     (108/385.d0*x2**2+40/77.d0*x4+36/35.d0*x2)*bb2 +
     &     (80/77.d0*x2+1.d0)*bb4 
      
      a6(m0,x2,aa2,bb2,x4,aa4,bb4,aa6,bb6)= 
     &     m0*(-10/33.d0*x4-6/77.d0*x2**2)+
     &     (10/11.d0*x2*x4-18/77.d0*x2**3)+
     &     (-15/11.d0*x4+108/77.d0*x2**2)*aa2+
     &     (-25/11.d0*x2)*aa4+
     &     aa6+
     &     (18/77.d0*x2**2+10/11.d0*x4)*bb2+
     &     (20/11.d0*x2)*bb4+
     &     bb6
      
      f2(x2,x4)=6/175.d0*x2**3+24/35.d0*x2*x4+12/35.d0*x2**2+3/5.d0*x2
      f4(x2,x4)=36/77.d0*x2**3+40/77.d0*x2*x4+18/35.d0*x2**2+1/3.d0*x4
      f6(x2,x4,x6)=72/143.d0*x2**3+120/143.d0*x2*x4+3/13.d0*x6
      
      h0(x2,x4)=-2/35.d0*x2**3-3/10.d0*x2**2
      g2(x2,x4)=36/175.d0*x2**3-6/35.d0*x2*x4+3/5.d0*x2-3/35.d0*x2**2
      g4(x2,x4)=72/385.d0*x2**3-20/77.d0*x2*x4-9/35.d0*x2**2+1/3.d0*x4
      g6(x2,x4,x6)=270/1001.d0*x2**3-75/143.d0*x2*x4+3/13.d0*x6
      
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c     Ordre des splines pour l'integration 
      data m_Sf/4/

c     Changement maximal pour convergence des s2, s4
      real*8 maxchg
      data maxchg/0.5d0/

      data prec/1d-8/,precif/1.d-6/

c...........S.............A...........V..............E..................
      save 
c-----------------------------------------------------------------------
 2000 format(1p8d10.3)
      mm=vrot**2/(4/3.d0*pi*g)
      romoy=mtot/(4/3.d0*pi*stot**3)
c      write(*,122)stot,mtot,romoy,vrot,mm,vrot**2*stot**3/g/mtot
 122  format(1x,'stot=',1pd12.5,' mtot=',1pd12.5,' romoy=',1pd12.5,
     &     ' vrot=',1pd12.5,' mm=',1pd12.5,' m=',1pd12.5)
      
      if (old_a1.eq.0.d0) then
c      if (.true.) then
       do i=1,nn-1
        ss0(i)=romoy*m(i)/s(i)/s(i)/s(i)
       enddo
       ss0(nn)=ro(nn)
       do i=1,nn
        mm0(i)=mm/ss0(i)
        s2(i)=-mm0(i)/3.d0
       enddo
       old_j2=0.d0
       old_j4=0.d0
       old_j6=0.d0
      endif
      
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Convergence sur s2, s4, s6                                       c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      er=1.d10
      iter=0
      do while ((er.gt.precif).and.(iter.lt.iter_max))
       iter=iter+1
       do i=1,nn
        err=1.d10
        iter2=0
        do while ((err.gt.prec).and.(iter2.lt.iter_max))
         new_s2=a2(mm0(i),s2(i),ss2(i),tt2(i),s4(i),ss4(i),tt4(i))
         new_s4=a4(mm0(i),s2(i),ss2(i),tt2(i),s4(i),ss4(i),tt4(i))
         err2=abs((new_s2-s2(i))/new_s2)
         if ((err2.gt.maxchg).and.(abs(s2(i)).gt.1d-8)) then
          if (new_s2.gt.s2(i)) then
           s2(i)=s2(i)/(1-maxchg)
          else
           s2(i)=s2(i)/(1+maxchg)
          endif
         else
          s2(i)=new_s2
         endif
         err4=abs((new_s4-s4(i))/new_s4)
         if ((err4.gt.maxchg).and.(abs(s4(i)).gt.1d-8)) then
          if (new_s4.gt.s4(i)) then
           s4(i)=s4(i)/(1-maxchg)
          else
           s4(i)=s4(i)/(1+maxchg)
          endif
         else
          s4(i)=new_s4
         endif
         err=max(err2,err4)
         iter2=iter2+1
        enddo
        s6(i)=a6(mm0(i),s2(i),ss2(i),tt2(i),s4(i),ss4(i),tt4(i),
     &       ss6(i),tt6(i))
       enddo
       

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     write(*,*)'Integration des ss'
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
       call integre_ss3(m_Sf,nn,s,dros,ndis,idisc,s2,ss2,s4,ss4,s6,ss6)
c     --- discontinuites ----
       do i=1,ndis
        do j=idisc(i)-1,1,-1
         ss2(j)=ss2(j)-s(idisc(i)-1)**5*f2(s2(idisc(i)-1),
     &        s4(idisc(i)-1))*deltaro(i)
         ss4(j)=ss4(j)-s(idisc(i)-1)**7*f4(s2(idisc(i)-1),
     &        s4(idisc(i)-1))*deltaro(i)
         ss6(j)=ss6(j)-s(idisc(i)-1)**9*f6(s2(idisc(i)-1),
     &        s4(idisc(i)-1),s6(idisc(i)-1))*deltaro(i)
        enddo
       enddo
c     -----------------------
       do i=1,nn-1
        ss2(i)=(f2(s2(i),s4(i))*ro(i)-ss2(i)/s(i)**5)/ss0(i)
        ss4(i)=(f4(s2(i),s4(i))*ro(i)-ss4(i)/s(i)**7)/ss0(i)
        ss6(i)=(f6(s2(i),s4(i),s6(i))*ro(i)-ss6(i)/s(i)**9)/ss0(i)
       enddo
       ss2(nn)=f2(s2(nn),s4(nn))*ro(nn)/ss0(nn)
       ss4(nn)=f4(s2(nn),s4(nn))*ro(nn)/ss0(nn)
       ss6(nn)=f6(s2(nn),s4(nn),s6(nn))*ro(nn)/ss0(nn)


cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     write(*,*)'Integration des tt'
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc       
       call integre_tt3(m_Sf,nn,s,dros,ndis,idisc,s2,tt2,s4,tt4,s6,tt6)
c     --- discontinuites ----
       do i=1,ndis
        do j=idisc(i),nn
         tt2(j)=tt2(j)-g2(s2(idisc(i)),s4(idisc(i)))*deltaro(i)
         tt4(j)=tt4(j)-1/s(idisc(i))**2*g4(s2(idisc(i)),
     &        s4(idisc(i)))*deltaro(i)
         tt6(j)=tt6(j)-1/s(idisc(i))**4*g6(s2(idisc(i)),
     &        s4(idisc(i)),s6(idisc(i)))*deltaro(i)
        enddo
       enddo
c     -----------------------
       do i=2,nn
        tt2(i)=(-g2(s2(i),s4(i))*ro(i)-tt2(i))/ss0(i)
        tt4(i)=(-g4(s2(i),s4(i))*ro(i)-tt4(i)*s(i)**2)/ss0(i)
        tt6(i)=(-g6(s2(i),s4(i),s6(i))*ro(i)-tt6(i)*s(i)**4)/ss0(i)
       enddo
       tt2(1)=0.d0
       tt4(1)=0.d0
       tt6(1)=0.d0
       
       a1=stot*req(s2(1),s4(1),s6(1))+1.d-99
       j2=-(stot/a1)**2*ss2(1)+1.d-99
       j4=-(stot/a1)**4*ss4(1)+1d-99
       j6=-(stot/a1)**6*ss6(1)+1d-99
       er=max(abs((a1-old_a1)/a1),abs((j2-old_j2)/j2),
     &      abs((j4-old_j4)/j4),abs((j6-old_j6)/j6))
       old_a1=a1
       old_j2=j2
       old_j4=j4
       old_j6=j6
c       write(*,300)iter,er,a1,j2,j4,j6
 300   format(1x,'iter_fig3:',i3,1x,i3,' err.max:',1p,d8.1,' a1=',
     &      d12.5,' j2=',d12.5,' j4=',d12.5,
     &      ' j6=',d12.5)
c     pause 'FIGURES3: pause'
      enddo
      
      if (silent.eq.0) write(*,300)iter,iter2,er,a1,j2,j4,j6
      
      if (max(iter,iter2).eq.iter_max) then
       write(*,*)'FIGURES3: pas de convergence - je garde la ',
     &      'solution precedente'
       ierr=1
       init=.true.
       return
      endif

      ierr=0
      do i=1,nn
       b(i)=ss0(i)/romoy*b0(mm0(i),s2(i),ss2(i),tt2(i))
       h(i)=h0(s2(i),s4(i))/romoy
      enddo
      
      if (silent.lt.0) then
       write(*,*)' ****** FIGURES3: fonctions poids *******'
       open(unit=22,file='contrib3.dat',form='formatted',
     &      status='unknown')
       write(22,*)nn
       do i=1,nn-1
        write(22,'(i4,1p8d15.7)')i,s(i),ss0(i),ss2(i),
     &       ss4(i),ss6(i),0.
       enddo
       write(22,'(1p8d15.7)')s(nn),0.,0.,0.,0.,0.
       write(*,*)'J''ai cree le fichier contrib3.dat'
       close(22)

       open(unit=22,file='bid3.dat',form='formatted',status='unknown')
       write(22,*)nn
       do i=1,nn
        write(22,201)i,m(i),s(i),s0(s2(i),s4(i)),s2(i),s4(i),s6(i),
     &       b(i),h(i)
 201    format(i4,1p,8d13.5)
       enddo
       write(*,*)'J''ai cree le fichier "bid3.dat"'
       close(22)

       open(unit=22,file='bid3s.dat',status='unknown')
       write(22,222)ndis,(idisc(i),i=1,5)
       do i=1,nn
        write(22,223)i,s(i),ss0(i),ss2(i),ss4(i),ss6(i)
       enddo
 222   format(1x,'ndis=',i4,/,1x,'idisc=',5(1x,i4),/)
 223   format(1x,i3,1x,6(1pd12.5))
       write(*,*)'J''ai cree le fichier "bid3s.dat"'
       close(22)
       
       open(unit=22,file='bid3t.dat',status='unknown')
       write(22,222)ndis,(idisc(i),i=1,5)
       do i=1,nn
        write(22,223)i,s(i),tt2(i),tt4(i),tt6(i)
       enddo
       write(*,*)'J''ai cree le fichier "bid3t.dat"'
       close(22)

      endif

      return
      
      end
