      
c******************************************************************
      
      subroutine figures4(mtot,stot,vrot,m,s,ro,dros,nn,ndis,idisc,
     &     deltaro,a1,j2,j4,j6,j8,b,h,ierr)
c     Version: 00/05/03
c     l'ordre 4 
      
c     Auteur: Tristan Guillot
      
c-Entrees:
c     mtot: masse totale
c     stot: rayon total
c     vrot: vitesse angulaire de rotation [rad/s]
c     m: tableau des masses (normalises - m(1)=1.; m(nn)=0.)
c     s: tableau des rayons moyens (normalises - r(1)=1.; r(nn)=0.)
c     ro: densites en g/cm3
c     dros: d(ro)/ds  [g/cm3]
c     nn: nombre de couches du modele
c     ndis: nombre de discontinuites (min=1)
c     idisc: tableau des couches des discontinuites
c     (couche inferieure: idisc(idis)-1; couche superieure: idisc(idis))
c     deltaro: tableau des sauts de densite (densite couche sup - couche inf)
c-Sorties:
c     a1: rayon equatorial
c     j2,j4,j6,j8: potentiels gravitationnels
c     b,h: tableaux des corrections au potentiel dues a la rotation
c     ierr: 0: tvb; 1: pb de convergence
      
c---- Exemple de discontinuite: ------------------------------------------
c     idis=1 -> transition MIP/surface
c     idis=2: (par ex. PPT)
c     couche      m           s        ro
c     76      0.87185    0.96172   0.91753
c     77      0.86342    0.95618   0.93545   <- couche "inferieure"
c     78      0.86342    0.95618   1.30445   <- couche "superieure"
c     79      0.85950    0.95121   1.31789
c     ==> idisc(2)=78; deltaro(2)=+0.36900
c--------------------------------------------------------------------------
      
      implicit none
      
      include './cepam.inc'
      integer pn_d
      parameter (pn_d=pn+pn_noy+pndis)
      
      integer nn,ndis,idisc(ndis),ierr
      real*8 mtot,stot,vrot,m(nn),s(nn),ro(nn),dros(nn),
     &     deltaro(ndis),b(nn),h(nn)
      
      integer i,j,iter,ncouche,m_Sf
      real*8 ss0(pn_d),mm0(pn_d),ss0_3(pn_d),
     &     v0,a1,old_a1,ds,mm,er,err,ss,romoy,
     &     s2(pn_d),ss2(pn_d),tt2(pn_d),j2,new_s2,old_j2,
     &     s4(pn_d),ss4(pn_d),tt4(pn_d),j4,new_s4,old_j4,
     &     s6(pn_d),ss6(pn_d),tt6(pn_d),j6,new_s6,old_j6,
     &     s8(pn_d),ss8(pn_d),tt8(pn_d),j8,new_s8,old_j8
      real*8 aerr0,prec,precif
      data prec/1d-8/,precif/1.d-4/
      
      logical init,verif
      data init/.true./,verif/.false./
      
      include './Communs/modelp.inc'
      include './Communs/ctephys.inc'
     
ccccccccccccccccEquations de la theorie des figures cccccccccccccccccc
      
      real*8 s0,req,b0,f0,h0,a2,f2,g2,a4,f4,g4,a6,f6,g6,a8,f8,g8
      real*8 m0,x2,aa2,bb2,x4,aa4,bb4,x6,aa6,bb6,x8,aa8,bb8
      
      s0(x2,x4)=-1/5.d0*x2**2-2/105.d0*x2**3-1/35.d0*x2**4
     &     -2/35.d0*x2**2*x4-1/9.d0*x4**2
      req(x2,x4,x6,x8)=1+s0(x2,x4)-1/2.d0*x2+3/8.d0*x4-5/16.d0*x6
     &     +35/128.d0*x8
      
      b0(m0,x2,aa2,bb2,x4,aa4,bb4)=
     &     m0*(-2/15.d0*x2-3/35.d0*x2**2+22/1575.d0*x2**3
     &     -4/105.d0*x2*x4)+
     &     (2/5.d0*x2**2-4/105.d0*x2**3+48/175.d0*x2**4
     &     -4/35.d0*x2**2*x4+2/9.d0*x4**2)+
     &     (-3/5.d0*x2+12/35.d0*x2**2-234/175.d0*x2**3
     &     +24/35.d0*x2*x4)*aa2+
     &     (192/35.d0*x2**2-5/9.d0*x4)*aa4+
     &     (2/5.d0*x2+2/35.d0*x2**2-2/35.d0*x2**3+4*x2*x4/35.d0)*bb2+
     &     (12/35.d0*x2**2+4/9.d0*x4)*bb4
      
      a2(m0,x2,aa2,bb2,x4,aa4,bb4)=m0*(-1/3.d0+10/21.d0*x2
     &     +3/35.d0*x2**2
     &     -26/315.d0*x2**3-4/21.d0*x4+20/231.d0*x2*x4)+
     &     (2/7.d0*x2**2-29/35.d0*x2**3+454/1155.d0*x2**4+4/7.d0*x2*x4
     &     -36/77.d0*x2**2*x4+100/693.d0*x4**2)+
     &     (1-6/7.d0*x2+111/35.d0*x2**2-1242/385.d0*x2**3-6/7.d0*x4
     &     +144/77.d0*x2*x4)*aa2 +
     &     (-10/7.d0*x2+1152/77.d0*x2**2-500/693.d0*x4)*aa4 +
     &     (1+4/7.d0*x2+1/35.d0*x2**2-16/105.d0*x2**3+4/7.d0*x4
     &     +24/77.d0*x2*x4)*bb2 +
     &     (8/7.d0*x2+72/77.d0*x2*x2+400/693.d0*x4)*bb4
      
      a4(m0,x2,aa2,bb2,x4,aa4,bb4,x6,aa6,bb6)=
     &     m0*(-12/35.d0*x2+6/77.d0*x2**2
     &     +12/175.d0*x2**3+38/77.d0*x4-326/5005.d0*x2*x4
     &     -30/143.d0*x6)+
     &     (18/35.d0*x2**2-108/385.d0*x2**3+16902/25025.d0*x2**4
     &     +40/77.d0*x2*x4-7369/5005.d0*x2**2*x4+162/1001.d0*x4**2
     &     +90/143.d0*x2*x6)+
     &     (-54/35.d0*x2+648/385.d0*x2**2-122688/25025.d0*x2**3
     &     -60/77.d0*x4+21468/5005.d0*x2*x4-135/143.d0*x6)*aa2 +
     &     (1-100/77.d0*x2+176749/5005.d0*x2**2-810/1001.d0*x4)*aa4 +
     &     (315/143.d0*x2)*aa6 +
     &     (36/35.d0*x2+108/385.d0*x2**2-36/175.d0*x2**3+40/77.d0*x4
     &     +3578/5005.d0*x2*x4+90/143.d0*x6)*bb2 +
     &     (1+80/77.d0*x2+1346/1001.d0*x2**2+648/1001.d0*x4)*bb4 +
     &     (270/143.d0*x2)*bb6
      
      a6(m0,x2,aa2,bb2,x4,aa4,bb4,x6,aa6,bb6)= 
     &     m0*(-6/77.d0*x2**2-10/33.d0*x4+34/231.d0*x2*x4
     &     +82/165.d0*x6)+
     &     (-18/77.d0*x2**3+72/385.d0*x2**4+10/11.d0*x2*x4
     &     -54/77.d0*x2**2*x4+20/99.d0*x4**2+28/55.d0*x2*x6)+
     &     (108/77.d0*x2**2-144/77.d0*x2**3-15/11.d0*x4+216/77.d0*x2*x4
     &     -42/55.d0*x6)*aa2+
     &     (-25/11.d0*x2+1728/77.d0*x2**2-100/99.d0*x4)*aa4+
     &     (1-98/55.d0*x2)*aa6+
     &     (18/77.d0*x2**2+10/11.d0*x4+36/77.d0*x2*x4+28/55.d0*x6)*bb2+
     &     (20/11.d0*x2+108/77.d0*x2**2+80/99.d0*x4)*bb4+
     &     (1+84/55.d0*x2)*bb6
      
      a8(m0,x2,aa2,bb2,x4,aa4,bb4,x6,aa6,bb6,x8,aa8,bb8)= 
     &     m0*(-56/429.d0*x2*x4-56/195.d0*x6)+
     &     (72/715.d0*x2**4-84/143.d0*x2**2*x4+490/1287.d0*x4**2
     &     +56/65.d0*x2*x6)+
     &     (-144/143.d0*x2**3+336/143.d0*x2*x4-84/65.d0*x6)*aa2+
     &     (2688/143.d0*x2**2-2450/1287.d0*x4)*aa4+
     &     (196/65.d0*x2)*aa6+
     &     aa8+
     &     (56/143.d0*x2*x4+56/65.d0*x6)*bb2+
     &     (168/143.d0*x2**2+1960/1287.d0*x4)*bb4+
     &     (168/65.d0*x2)*bb6+
     &     bb8
      
      f0(x2)=-3/35.d0*x2**4
c      f0(x2)=0.d0
      f2(x2,x4)=3/5.d0*x2+12/35.d0*x2**2+6/175.d0*x2**3
     &     -184/1925.d0*x2**4
     &     +24/35.d0*x2*x4+216/385.d0*x2**2*x4+40/2321.d0*x4**2
      f4(x2,x4,x6)=18/35.d0*x2**2+36/77.d0*x2**3+486/5005.d0*x2**4
     &     +1/3.d0*x4+40/77.d0*x2*x4+6943/5005.d0*x2**2*x4
     &     +162/1001.d0*x4**2+90/143.d0*x2*x6
      f6(x2,x4,x6)=72/143.d0*x2**3+432/715.d0*x2**4+120/143.d0*x2*x4
     &     +216/143.d0*x2**2*x4+80/429.d0*x4**2+3/13.d0*x6
     &     +336/715.d0*x2*x6
      f8(x2,x4,x6,x8)=1296/2431.d0*x2**4+3780/2431.d0*x2**2*x4
     &     +2450/7293.d0*x4**2+168/221.d0*x2*x6+3/17.d0*x8
      
      h0(x2,x4)=-3/10.d0*x2**2-2/35.d0*x2**3-9/350.d0*x2**4
     &     -6/35.d0*x2**2*x4-1/6.d0*x4**2
      g2(x2,x4)=3/5.d0*x2-3/35.d0*x2**2+36/175.d0*x2**3-17/275.d0*x2**4
     &     -6/35.d0*x2*x4+36/385.d0*x2**2*x4-10/231.d0*x4**2
      g4(x2,x4,x6)=-9/35.d0*x2**2+72/385.d0*x2**3
     &     +1062/2275.d0*x2**4+1/3.d0*x4
     &     -20/77.d0*x2*x4+4579/5005.d0*x2**2*x4-81/1001.d0*x4**2
     &     -45/143.d0*x2*x6
      g6(x2,x4,x6)=270/1001.d0*x2**3-54/143.d0*x2**4-75/143.d0*x2*x4
     &     +810/1001.d0*x2**2*x4-50/429.d0*x4**2+3/13.d0*x6
     &     -42/143.d0*x2*x6
      g8(x2,x4,x6,x8)=-4536/12155.d0*x2**4+2352/2431.d0*x2**2*x4
     &     -1715/7293.d0*x4**2-588/1105.d0*x2*x6+3/17.d0*x8
      
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      
c...........S.............A...........V..............E..................
      save 
c-----------------------------------------------------------------------
 2000 format(1p8d10.3)
      mm=vrot**2/(4/3.d0*pi*g)
      romoy=mtot/(4/3.d0*pi*stot**3)
c      write(*,222)stot,mtot,romoy,vrot,mm,vrot**2*stot**3/g/mtot
c 222  format(1x,'stot=',1pd12.5,' mtot=',1pd12.5,' romoy=',1pd12.5,
c     &     ' vrot=',1pd12.5,' mm=',1pd12.5,' m=',1pd12.5)
c      write(*,*)'nn,ndis,idisc',nn,ndis,idisc
      
      do i=1,nn-1
       ss0_3(i)=romoy*m(i)/s(i)/s(i)/s(i)
       ss0(i)=ss0_3(i)
       mm0(i)=mm/ss0(i)
      enddo
      ss0_3(nn)=ro(nn)
      ss0(nn)=ss0_3(nn)
      mm0(nn)=mm/ss0(nn)
c     write(*,*)'ss0(i)'
c     write(*,2000)(ss0(i),i=1,nn)
      
      if (init) then
       init=.false.
       do i=1,nn
        s2(i)=-mm/3.d0/ss0(i)
       enddo
       old_a1=0.d0
       old_j2=0.d0
       old_j4=0.d0
       old_j6=0.d0
       old_j8=0.d0
c       write(6,*)'nn=',nn,' Precif, Ordre des splines?'
c       read(*,*)precif,m_Sf
       precif=1.d-6
       m_Sf=4
      endif
      
      er=1.d10
      iter=0
      do while ((er.gt.precif).and.(iter.lt.iter_max))
       iter=iter+1
       do i=1,nn
        mm0(i)=mm/ss0(i)
       enddo
       do i=1,nn
        err=1.d10
        do while (err.gt.prec)
         new_s2=a2(mm0(i),s2(i),ss2(i),tt2(i),s4(i),ss4(i),tt4(i))
         new_s4=a4(mm0(i),s2(i),ss2(i),tt2(i),s4(i),ss4(i),tt4(i),
     &        s6(i),ss6(i),tt6(i))
         new_s6=a6(mm0(i),s2(i),ss2(i),tt2(i),s4(i),ss4(i),tt4(i),
     &        s6(i),ss6(i),tt6(i))
         err=max(abs((new_s2-s2(i))/new_s2),abs((new_s4-s4(i))
     &        /new_s4),abs((new_s6-s6(i))/new_s6))
         s2(i)=new_s2
         s4(i)=new_s4
         s6(i)=new_s6
        enddo
        err=1.d10
        do while (err.gt.prec)
         new_s8=a8(mm0(i),s2(i),ss2(i),tt2(i),s4(i),ss4(i),tt4(i),
     &        s6(i),ss6(i),tt6(i),s8(i),ss8(i),tt8(i))
         err=abs((new_s8-s8(i))/new_s8)
         s8(i)=new_s8
        enddo
       enddo
c     write(*,*)'s2(1),s4(1)=',s2(1),s4(1)
       
c     write(*,*)'Integration des ss'
       call integre_ss4(m_Sf,nn,s,dros,ndis,idisc,ss0,s2,ss2,s4,ss4,
     &      s6,ss6,s8,ss8)
c     --- discontinuites ----
       do i=1,ndis
        do j=idisc(i)-1,1,-1
         ss0(j)=ss0(j)-s(idisc(i)-1)**3*f0(s2(idisc(i)-1))
     &        *deltaro(i)
c         ss0(j)=ss0(j)-deltaro(i)
         ss2(j)=ss2(j)-s(idisc(i)-1)**5*f2(s2(idisc(i)-1),
     &        s4(idisc(i)-1))*deltaro(i)
         ss4(j)=ss4(j)-s(idisc(i)-1)**7*f4(s2(idisc(i)-1),
     &        s4(idisc(i)-1),s6(idisc(i)-1))*deltaro(i)
         ss6(j)=ss6(j)-s(idisc(i)-1)**9*f6(s2(idisc(i)-1),
     &        s4(idisc(i)-1),s6(idisc(i)-1))*deltaro(i)
         ss8(j)=ss8(j)-s(idisc(i)-1)**11*f8(s2(idisc(i)-1),
     &        s4(idisc(i)-1),s6(idisc(i)-1),s8(idisc(i)-1))
     &        *deltaro(i)
        enddo
       enddo
c     -----------------------
       do i=1,nn-1
c        if (ro(nn)-ro(i).le.ro(nn)*5d-3) then
c         ss0(i)=(ro(nn)+ro(i))/2.d0*f0(s2(i))
c        else
        ss0(i)=ss0_3(i)+(f0(s2(i))*ro(i)-ss0(i)/s(i)**3)
c        endif
c        ss0(i)=ro(i)-ro(nn)-ss0(i)
        ss2(i)=(f2(s2(i),s4(i))*ro(i)-ss2(i)/s(i)**5)/ss0(i)
        ss4(i)=(f4(s2(i),s4(i),s6(i))*ro(i)-ss4(i)/s(i)**7)/ss0(i)
        ss6(i)=(f6(s2(i),s4(i),s6(i))*ro(i)-ss6(i)/s(i)**9)/ss0(i)
        ss8(i)=(f8(s2(i),s4(i),s6(i),s8(i))*ro(i)-ss8(i)/s(i)**11)
     &       /ss0(i)
       enddo
       ss0(nn)=ss0_3(nn)+f0(s2(nn))*ro(nn)
c       ss0(nn)=-ss0(nn)
       ss2(nn)=f2(s2(nn),s4(nn))*ro(nn)/ss0(nn)
       ss4(nn)=f4(s2(nn),s4(nn),s6(nn))*ro(nn)/ss0(nn)
       ss6(nn)=f6(s2(nn),s4(nn),s6(nn))*ro(nn)/ss0(nn)
       ss8(nn)=f8(s2(nn),s4(nn),s6(nn),s8(nn))*ro(nn)/ss0(nn)
       
c     write(*,*)'Integration des tt'
       call integre_tt4(m_Sf,nn,s,dros,ndis,idisc,s2,tt2,s4,tt4,
     &      s6,tt6,s8,tt8)
c     --- discontinuites ----
       do i=1,ndis
        do j=idisc(i),nn
         tt2(j)=tt2(j)-g2(s2(idisc(i)),s4(idisc(i)))*deltaro(i)
         tt4(j)=tt4(j)-1/s(idisc(i))**2*g4(s2(idisc(i)),
     &        s4(idisc(i)),s6(idisc(i)))*deltaro(i)
         tt6(j)=tt6(j)-1/s(idisc(i))**4*g6(s2(idisc(i)),
     &        s4(idisc(i)),s6(idisc(i)))*deltaro(i)
         tt8(j)=tt8(j)-1/s(idisc(i))**6*g8(s2(idisc(i)),
     &        s4(idisc(i)),s6(idisc(i)),s8(idisc(i)))*deltaro(i)
        enddo
       enddo
c     -----------------------
       do i=2,nn
        tt2(i)=(-g2(s2(i),s4(i))*ro(i)-tt2(i))/ss0(i)
        tt4(i)=(-g4(s2(i),s4(i),s6(i))*ro(i)-tt4(i)*s(i)**2)/ss0(i)
        tt6(i)=(-g6(s2(i),s4(i),s6(i))*ro(i)-tt6(i)*s(i)**4)/ss0(i)
        tt8(i)=(-g8(s2(i),s4(i),s6(i),s8(i))*ro(i)-tt8(i)*s(i)**6)
     &       /ss0(i)
       enddo
       tt2(1)=0.d0
       tt4(1)=0.d0
       tt6(1)=0.d0
       tt8(1)=0.d0
       
       a1=stot*req(s2(1),s4(1),s6(1),s8(1))
       j2=-(stot/a1)**2*ss2(1)
       j4=-(stot/a1)**4*ss4(1)
       j6=-(stot/a1)**6*ss6(1)+1d-99
       j8=-(stot/a1)**8*ss8(1)+1d-99
       er=max(abs((a1-old_a1)/a1),abs((j2-old_j2)/j2),
     &      abs((j4-old_j4)/j4),abs((j6-old_j6)/j6),
     &      abs((j8-old_j8)/j8))
c       er=0.d0
       old_a1=a1
       old_j2=j2
       old_j4=j4
       old_j6=j6
       old_j8=j8
c       write(*,300)iter,er,a1,j2,j4,j6,j8
 300   format(1x,'iter_fig4:',i3,' err.max:',1p,d8.1,' a1=',
     &      d12.5,' j2=',d12.5,' j4=',d12.5,
     &      ' j6=',d12.5,' j8=',d12.5)
c     pause 'FIGURES4: pause'
      enddo
      
      if (silent.eq.0) write(*,300)iter,er,a1,j2,j4,j6,j8
      
      if (iter.eq.iter_max) then
       write(*,*)'FIGURES4: pas de convergence - je garde la ',
     &      'solution precedente'
       ierr=1
       init=.true.
       return
      endif

      ierr=0
      do i=1,nn
       if (ro(nn)-ro(i).gt.ro(nn)*1d-2) then
        b(i)=ss0(i)/romoy*(b0(mm0(i),s2(i),ss2(i),tt2(i),s4(i),ss4(i),
     &       tt4(i))+1.d0)-m(i)/s(i)**3
       else
        b(i)=ss0(i)/romoy*(b0(mm0(i),s2(i),ss2(i),tt2(i),s4(i),ss4(i),
     &       tt4(i))+1.d0)-(ro(nn)+ro(i))/2.d0/romoy
       endif
       h(i)=h0(s2(i),s4(i))/romoy
c       h(i)=mm/romoy*(-2/15.d0*s2(i))
      enddo
      
c      open(unit=22,file='bid4.dat',form='formatted',status='unknown')
c      write(22,*)nn
c      do i=1,nn
c       write(22,201)m(i),s(i),s0(s2(i),s4(i)),s2(i),s4(i),s6(i)
cc       write(22,200)i,s(i),ss0(i)-romoy*m(i)/s(i)/s(i)/s(i),
cc     &      s2(i),ss2(i),tt2(i)
cc     &      ,s4(i),ss4(i),tt4(i),s6(i),ss6(i),tt6(i),s8(i),ss8(i),tt8(i)
cc 200   format(i4,1x,14(1x,1pd10.3))
c 201   format(14(1x,1pd12.5))
c      enddo
cc      i=nn
cc      write(22,200)i,s(i),(ss0(i)-ro(nn)),
cc     &     s2(i),ss2(i),tt2(i)
c      write(*,*)'J''ai cree le fichier BID4.DAT'
c      close(22)

c      write(*,*)' ****** FIGURES4: fonctions poids *******'
c      open(unit=22,file='contrib4.dat',form='formatted',
c     &     status='unknown')
c      write(22,*)nn
c      do i=1,nn-1
c       write(22,'(1p8d15.7)')s(i),ss0(i),ss2(i),
c     &      ss4(i),ss6(i),ss8(i)
c       write(*,'(1p8d15.7)')s(i),f0(s2(i)),f2(s2(i),s4(i)),
c     &      f4(s2(i),s4(i),s6(i)),f6(s2(i),s4(i),s6(i))
c      enddo
c      write(22,'(1p8d15.7)')s(nn),0.,0.,0.,0.,0.
c      write(*,*)'J''ai cree le fichier contrib4.dat'
c      close(22)
 
c       write(22,201)m(i),s(i),s0(s2(i),s4(i)),s2(i),s4(i),s6(i)
cc       write(22,200)i,s(i),ss0(i)-romoy*m(i)/s(i)/s(i)/s(i),
cc     &      s2(i),ss2(i),tt2(i)
cc     &      ,s4(i),ss4(i),tt4(i),s6(i),ss6(i),tt6(i),s8(i),ss8(i),tt8(i)
cc 200   format(i4,1x,14(1x,1pd10.3))
c 201   format(14(1x,1pd12.5))
c      enddo
cc      i=nn
cc      write(22,200)i,s(i),(ss0(i)-ro(nn)),
cc     &     s2(i),ss2(i),tt2(i)
c      write(*,*)'J''ai cree le fichier BID4.DAT'
c      close(22)

      
      return
      
      end
