      
c******************************************************************
      
      subroutine figures2(mtot,stot,vrot,m,s,ro,dros,nn,ndis,idisc,
     &     deltaro,a1,j2,j4,b,h,ierr)
c     Version: 00/05/03
c     Calculs du potentiel centrifuge par la theorie des figures a l'ordre 2
      
c     Auteur: Tristan Guillot
      
c Entrees:
c     mtot: masse totale
c     stot: rayon total
c     vrot: vitesse angulaire de rotation [rad/s]
c     m: tableau des masses (normalises - m(1)=1.; m(nn)=0.)
c     s: tableau des rayons moyens (normalises - r(1)=1.; r(nn)=0.)
c     ro: densites en g/cm3
c     dros: d(ro)/ds  [g/cm3]
c     nn: nombre de couches du modele
c     ndis: nombre de discontinuites (min=1)
c     idisc: tableau des couches des discontinuites
c            (couche inferieure: idisc(idis)-1; couche superieure: idisc(idis))
c     deltaro: tableau des sauts de densite (densite couche sup - couche inf)
c Sorties:
c     a1: rayon equatorial
c     j2,j4: potentiels gravitationnels
c     b(nn),h(nn): tableaux des corrections au potentiel dues a la rotation
c     ierr: 0: tvb; 1: pb de convergence

c---- Exemple de discontinuite: ------------------------------------------
c     idis=1 -> transition MIP/surface
c     idis=2: (par ex. PPT)
c           couche      m           s        ro
c             76      0.87185    0.96172   0.91753
c             77      0.86342    0.95618   0.93545   <- couche "inferieure"
c             78      0.86342    0.95618   1.30445   <- couche "superieure"
c             79      0.85950    0.95121   1.31789
c     ==> idisc(2)=78; deltaro(2)=+0.36900
c--------------------------------------------------------------------------

c-Remarques:
c     "figures2" est appele avant "figures3" pour initialisation
c     du COMMON /fig2/

      implicit none
      
      include './cepam.inc'
      integer pn_d
      parameter (pn_d=pn+pn_noy+pndis)
      
      integer nn,ndis,idisc(ndis),ierr
      real*8 mtot,stot,vrot,m(nn),s(nn),ro(nn),dros(nn),
     &     deltaro(ndis),b(nn),h(nn)
      
      integer i,j,iter,iter2
      real*8 ss0(pn_d),mm0(pn_d),
     &     a1,old_a1,mm,er,err,romoy,
     &     s2(pn_d),ss2(pn_d),tt2(pn_d),j2,new_s2,old_j2,
     &     s4(pn_d),ss4(pn_d),tt4(pn_d),j4,old_j4
      real*8 prec,precif
      
      logical init
      
      include './Communs/modelp.inc'
      include './Communs/ctephys.inc'

      common /fig2/old_a1,ss0,s2,ss2,tt2,old_j2,s4,ss4,tt4,old_j4

c      save /fig2/
c      save init
      
cccccccccccccccc Equations de la theorie des figures cccccccccccccccccc
      
      real*8 s0,req,b0,h0,a2,f2,g2,a4,f4,g4
      real*8 m0,x2,aa2,bb2,x4,aa4,bb4
      
      s0(x2,x4)=-1/5.d0*x2**2-2/105.d0*x2**3-1/35.d0*x2**4
     &      -2/35.d0*x2**2*x4-1/9.d0*x4**2
      req(x2,x4)=1.d0+s0(x2,x4)-1/2.d0*x2+3/8.d0*x4

      b0(m0,x2,aa2,bb2)=
     &     m0*(-2/15.d0*x2)+
     &     (2/5.d0*x2**2)+
     &     (-3/5.d0*x2)*aa2+
     &     (2/5.d0*x2)*bb2

      a2(m0,x2,aa2,bb2)=m0*(-1/3.d0+10/21.d0*x2)+
     &     (2/7.d0*x2**2)+
     &     (1.d0-6/7.d0*x2)*aa2 +
     &     (1.d0+4/7.d0*x2)*bb2
      
      a4(m0,x2,aa2,bb2,aa4,bb4)=m0*(-12/35.d0*x2)+
     &     (18/35.d0*x2**2) +
     &     (-54/35.d0*x2)*aa2 +
     &     aa4 +
     &     (36/35.d0*x2)*bb2 +
     &     bb4
      
      f2(x2)=3/5.d0*x2+12/35.d0*x2**2
      f4(x2,x4)=18/35.d0*x2**2+1/3.d0*x4
      
      h0(x2)=-3/10.d0*x2**2
      g2(x2)=3/5.d0*x2-3/35.d0*x2**2
      g4(x2,x4)=-9/35.d0*x2**2+1/3.d0*x4

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c     Ordre des splines pour l'integration 
      data m_S/4/

      data iter_max/50/
      data prec/1d-8/,precif/1.d-6/
      data init/.true./
      
c...........S.............A...........V..............E..................
      save 
c-----------------------------------------------------------------------
 2000 format(1p8d10.3)
      mm=vrot**2 /(4/3.d0*pi*g)
      romoy=mtot/(4/3.d0*pi*stot**3)
      write(*,222)stot,mtot,romoy,vrot,mm,mm/romoy
c      write(*,*)vrot**2*stot**3/g/mtot
 222  format(1x,'stot=',1pd12.5,' mtot=',1pd12.5,' romoy=',1pd12.5,
     &     ' vrot=',1pd12.5,/,' m*romoy=',1pd12.5,' m=',1pd12.5)

      do i=1,nn-1
       ss0(i)=romoy*m(i)/s(i)/s(i)/s(i)
       mm0(i)=mm/ss0(i)
      enddo
      ss0(nn)=ro(nn)
      mm0(nn)=mm/ss0(nn)
c     write(*,*)'ss0(i)'
c     write(*,2000)(ss0(i),i=1,nn)
      
      if (init) then
       init=.false.
       do i=1,nn
        s2(i)=-mm0(i)/3.d0
       enddo
       old_a1=0.d0
       old_j2=0.d0
       old_j4=0.d0
      endif
      
      er=1.d10
      iter=0
      do while ((er.gt.precif).and.(iter.lt.iter_max))
       iter=iter+1
       do i=1,nn
        err=1.d10
        iter2=0
        do while ((err.gt.prec).and.(iter2.lt.iter_max))
         iter2=iter2+1
         new_s2=a2(mm0(i),s2(i),ss2(i),tt2(i))
         err=abs((new_s2-s2(i))/new_s2)
         s2(i)=new_s2
        enddo
        s4(i)=a4(mm0(i),s2(i),ss2(i),tt2(i),ss4(i),tt4(i))
       enddo
       

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     write(*,*)'Integration des ss'
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
       call integre_ss2(m_S,nn,s,dros,ndis,idisc,s2,ss2,s4,ss4)
c --- discontinuites ----
       do i=1,ndis
        do j=idisc(i)-1,1,-1
         ss2(j)=ss2(j)-s(idisc(i)-1)**5*f2(s2(idisc(i)-1))*deltaro(i)
         ss4(j)=ss4(j)-s(idisc(i)-1)**7*f4(s2(idisc(i)-1),
     &        s4(idisc(i)-1))*deltaro(i)
        enddo
       enddo
c -----------------------
       do i=1,nn-1
        ss2(i)=(f2(s2(i))*ro(i)-ss2(i)/s(i)**5)/ss0(i)
        ss4(i)=(f4(s2(i),s4(i))*ro(i)-ss4(i)/s(i)**7)/ss0(i)
       enddo
       ss2(nn)=f2(s2(nn))*ro(nn)/ss0(nn)
       ss4(nn)=f4(s2(nn),s4(nn))*ro(nn)/ss0(nn)
       

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     write(*,*)'Integration des tt'
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
       call integre_tt2(m_S,nn,s,dros,ndis,idisc,s2,tt2,s4,tt4)
c --- discontinuites ----
       do i=1,ndis
        do j=idisc(i),nn
         tt2(j)=tt2(j)-g2(s2(idisc(i)))*deltaro(i)
         tt4(j)=tt4(j)-1/s(idisc(i))**2*g4(s2(idisc(i)),
     &        s4(idisc(i)))*deltaro(i)
        enddo
       enddo
c -----------------------
       do i=1,nn
        tt2(i)=(-g2(s2(i))*ro(i)-tt2(i))/ss0(i)
        tt4(i)=(-g4(s2(i),s4(i))*ro(i)-tt4(i)*s(i)**2)/ss0(i)
       enddo
       tt2(1)=0.d0
       tt4(1)=0.d0
       
       a1=stot*req(s2(1),s4(1))+1.d-99
       j2=-(stot/a1)**2*ss2(1)+1.d-99
       j4=-(stot/a1)**4*ss4(1)+1.d-99
       er=max(abs((a1-old_a1)/a1),abs((j2-old_j2)/j2),
     &      abs((j4-old_j4)/j4))
       old_a1=a1
       old_j2=j2
       old_j4=j4
c       write(*,300)iter,er,a1,j2,j4
 300   format(1x,'iter_fig2:',i3,' err.max:',1p,d8.1,' a1=',
     &      d12.5,' j2=',d12.5,' j4=',d12.5)
c     pause 'FIGURES2: pause'
      enddo
      
      if (silent.eq.0) write(*,300)iter,er,a1,j2,j4
      
      if (iter.eq.iter_max) then
       write(*,*)'FIGURES3: pas de convergence - je garde la ',
     &      'solution precedente'
       ierr=1
       init=.true.
       return
      endif

      ierr=0

      do i=1,nn
       b(i)=ss0(i)/romoy*b0(mm0(i),s2(i),ss2(i),tt2(i))
       h(i)=h0(s2(i))/romoy
c       h(i)=mm/romoy*(-2/15.d0*s2(i))
      enddo
      
      open(unit=22,file='bid2.dat',form='formatted',status='unknown')
      write(22,*)nn
      do i=1,nn
       write(22,201)m(i),s(i),s0(s2(i),s4(i)),s2(i),s4(i),0.d0
c       write(22,200)i,s(i),ss0(i),s2(i),ss2(i),tt2(i)
c     &      ,s4(i),ss4(i),tt4(i)
 200   format(i4,1x,8(1x,1pd10.3))
 201   format(8(1x,1pd12.5))
      enddo
      write(*,*)'J''ai cree le fichier BID2.DAT'
      close(22)

      return
      
      end
